package com.myoa.engineering.crawl.shopping.crawlhandler;

import com.myoa.engineering.crawl.shopping.crawlhandler.parser.FmkoreaArticleParser;
import com.myoa.engineering.crawl.shopping.domain.entity.v2.Article;
import com.myoa.engineering.crawl.shopping.infra.client.fmkorea.FmkoreaBoardClient;
import com.myoa.engineering.crawl.shopping.service.ArticleCommandService;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

@Slf4j
@Component
public class FmkoreaCrawlHandler implements CrawlHandler {

    private final FmkoreaBoardClient fmkoreaBoardClient;
    private final FmkoreaArticleParser fmkoreaArticleParser;
    private final ArticleCommandService articleCommandService;

    public FmkoreaCrawlHandler(FmkoreaBoardClient fmkoreaBoardClient,
                               FmkoreaArticleParser fmkoreaArticleParser, ArticleCommandService articleCommandService) {
        this.fmkoreaBoardClient = fmkoreaBoardClient;
        this.fmkoreaArticleParser = fmkoreaArticleParser;
        this.articleCommandService = articleCommandService;
    }

    @Override
    public CrawlTarget getCrawlTarget() {
        return CrawlTarget.FMKOREA;
    }

    @Override
    public void handle() {

        String fakeHtml = fmkoreaBoardClient.getBoardHtml("/index.php", generateRequestParams(1, null));
        String cookie = FmkoreaFake430Resolver.resolveFake430(fakeHtml);


        String boardHtmlPage1 = fmkoreaBoardClient.getBoardHtml("/index.php", generateRequestParams(1, cookie));
        List<Article> parsedPage1 = fmkoreaArticleParser.parse(boardHtmlPage1);

        String boardHtmlPage2 = fmkoreaBoardClient.getBoardHtml("/index.php", generateRequestParams(2, cookie));
        List<Article> parsedPage2 = fmkoreaArticleParser.parse(boardHtmlPage2);

        List<Article> merged = Stream.of(parsedPage1, parsedPage2)
                                     .flatMap(List::stream)
                                     .map(e -> e.updateCrawlTarget(getCrawlTarget()))
                                     .toList();

        articleCommandService.upsert(merged);
    }

    private Map<String, String> generateRequestParams(int pageId, String cookie) {
        Map<String, String> params = new HashMap<>();
        params.put("mid", "hotdeal");
        params.put("page", String.valueOf(pageId));
        params.put("Cookie", cookie);
        return params;
    }
}
