package com.myoa.engineering.crawl.shopping.service.slack;

import com.myoa.engineering.crawl.shopping.configuration.slack.properties.SlackSecretProperties;
import com.slack.api.methods.MethodsClient;
import com.slack.api.methods.request.chat.ChatPostMessageRequest;
import com.slack.api.methods.response.chat.ChatPostMessageResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class UserNotifyService {

    private static final String SLACK_PROPERTIES_UNIT_NAME = "shopping-crawler";

    private final SlackSecretProperties.SlackSecretPropertiesUnit slackSecretProperties;
    private final MethodsClient methodsClient;

    public UserNotifyService(SlackSecretProperties slackSecretProperties,
                             MethodsClient methodsClient) {
        this.slackSecretProperties = slackSecretProperties.find(SLACK_PROPERTIES_UNIT_NAME);
        this.methodsClient = methodsClient;
    }

    public ChatPostMessageResponse notify(ChatPostMessageRequest request) {
        try {
            return methodsClient.chatPostMessage(request);
        } catch (Exception e) {
            log.warn("Failed. message: {}", request, e);
            ChatPostMessageResponse response = new ChatPostMessageResponse();
            response.setOk(false);
            return response;
        }
    }

    public ChatPostMessageRequest.ChatPostMessageRequestBuilder generateMessage(String message) {
        return ChatPostMessageRequest.builder()
                                     .channel(slackSecretProperties.getChannel())
                                     .username(slackSecretProperties.getUsername())
                                     .text(message);
    }
}
